/*
 * [ConnectorPanel.java]
 *
 * Summary: common code for all connector-drawing methods.
 *
 * Copyright: (c) 2011-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2011-01-29 initial version.
 */
package com.mindprod.connectors;

import com.mindprod.common18.ST;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import static java.lang.System.*;

/**
 * common code for all connector-drawing methods.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2011-01-29 initial version.
 * @since 2011-01-29
 */
class ConnectorPanel extends JPanel
    {
    /**
     * colour to represent a ground/shield pin
     */
    static final Color GROUND_COLOR = Color.BLACK;

    /**
     * color to represent empty space in a connector
     */
    static final Color HOLE_COLOR = new Color( 0xfff0e0 );

    /**
     * colour to represent a negative pin
     */
    static final Color NEGATIVE_COLOR = new Color( 0x458b00 ); // dull green

    /**
     * color used to draw the outline of a component
     */
    static final Color OUTLINE_COLOR = new Color( 0x003000 );

    /**
     * colour to represent a positive pin
     */
    static final Color POSITIVE_COLOR = new Color( 0xcd3333 ); // dull red

    /**
     * colour to represent an unused pin
     */
    static final Color UNCONNECTED_COLOR = Color.GRAY;

    /**
     * colour of USB3 connectors
     */
    static final Color USB3_BLUE = new Color( 0x2295d6 );

    /**
     * color to display the connection against
     */
    private static final Color MATTE_COLOUR = new Color( 0xe0e0e0 );

    /**
     * default transparent background colour for images.
     */
    private static final Color TRANSPARENT = new Color( 0x00ffffff, true );

    /**
     * font used to label connectors
     */
    private static final Font USUAL_FONT = new Font( "Dialog", Font.PLAIN, 200 );

    /**
     * height of the virtual image we construct in pixels
     */
    final int imageHeight;

    /**
     * width of the virtual image we construct in pixels
     */
    final int imageWidth;

    /**
     * constructor. define virtual image dimensions, typically 1000 x 10000
     *
     * @param imageWidth  width of the virtual image. Size we construct in RAM before transforms.
     * @param imageHeight width of the virtual image. Size we construct in RAM before transforms.
     */
    ConnectorPanel( final int imageWidth, final int imageHeight )
        {
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        Dimension d = new Dimension( imageWidth, imageHeight );
        this.setMaximumSize( d );
        this.setMinimumSize( d );
        this.setPreferredSize( d );
        setOpaque( false );
        setBackground( TRANSPARENT );
        }

    /**
     * calculate a location in a clock circle.
     *
     * @param x      location of center
     * @param y      location of center
     * @param radius radius of clock
     * @param time   number 0..12 to indicate angular position, can be fractional.
     *
     * @return point on the circumference corresponding to that "time".
     */
    static Point clockPoint( int x, int y, int radius, double time )
        {
        final double angle = time / 12 * 2 * Math.PI;
        return new Point( ( int ) ( x + Math.sin( angle ) * radius ), ( int ) ( y - Math.cos( angle ) * radius ) );
        }

    /**
     * Draw string centered on x,y in both x and y directions
     *
     * @param g2d  the graphics context
     * @param x    location of center of string.
     * @param y    location of center of string.
     * @param text string to display
     */
    static void drawCenteredString( final Graphics2D g2d, final String text, final int x, final int y )
        {
        final Font font = g2d.getFont();
        int width = g2d.getFontMetrics().stringWidth( text );
        final FontRenderContext fr = g2d.getFontRenderContext();
        final LineMetrics lm = font.getLineMetrics( text, fr );
        final int height = ( int ) ( lm.getAscent() * .76 + .5 )/* compensate for overstating */;
        // x,y is bottom left corner of text
        g2d.drawString( text, x - width / 2, y + height / 2 );
        }

    /**
     * display this connector on screen hugely magnified
     */
    void display()
        {
        JFrame f = new JFrame();
        f.setSize( 1000 + 32 + 5, 1000 + 32 + 5 );
        Container contentPane = f.getContentPane();
        contentPane.setLayout( new FlowLayout() );
        contentPane.add( this );
        contentPane.setBackground( MATTE_COLOUR );
        f.setDefaultCloseOperation( WindowConstants.EXIT_ON_CLOSE );
        f.validate();
        f.setVisible( true );
        }

    /**
     * common to all connectors, arrange for anti-alias, and other defaults
     *
     * @param g where to paint
     */
    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING,
                RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
        g2d.setRenderingHint( RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY );
        // smooth geometric shapes too
        g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON );
        g2d.setBackground( TRANSPARENT );
        g2d.setColor( OUTLINE_COLOR );
        g2d.setFont( USUAL_FONT );
        }

    /**
     * save this image as a PNG file in the current directory.
     *
     * @param desiredWidth imageWidth of png in pixels
     * @param pngname      name of png file, including .png extension.
     */
    public void save( final int desiredWidth, final String pngname )
        {
        final double scale = ( double ) desiredWidth / ( double ) imageWidth;
        final int desiredHeight = ( int ) ( imageHeight * scale );
        try
            {
            BufferedImage bufferedImage =
                    new BufferedImage( desiredWidth, desiredHeight, BufferedImage.TYPE_4BYTE_ABGR_PRE );
            Graphics2D g2d = bufferedImage.createGraphics();
            g2d.transform( AffineTransform.getScaleInstance( scale, scale ) );
            // redraw using our paintComponent
            paintComponent( g2d );
            // write out the BufferedImage as a PNG
            final File pngFile = new File( pngname );
            ImageIO.write( bufferedImage, "png", pngFile );
            }
        catch ( IOException e )
            {
            err.println( "image not saved successfully" );
            }
        }

    /**
     * save this image as a PNG file in the current directory.
     *
     * @param desiredWidth imageWidth of png in pixels
     */
    public void save( final int desiredWidth )
        {
        save( desiredWidth,
                ST.chopLeadingString( getClass().getName().toLowerCase(), "com.mindprod.connectors." ) + ".png" );
        }
    }
