/*
 * [RJ45Female.java]
 *
 * Summary: draw RJ45Female, Ethernet T568B.
 *
 * Copyright: (c) 2011-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 2011-01-29 initial version.
 */
package com.mindprod.connectors;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;

/**
 * draw RJ45Female, Ethernet T568B.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2011-01-29 initial version.
 * @since 2011-01-29
 */
class RJ45Female extends ConnectorPanel
    {
    /**
     * x of the first pin
     */
    private static final int FIRST_PIN = 108;

    /**
     * size of horizontal jog in outline of connector
     */
    private static final int H_JOG = 167;

    /**
     * how much in from the edge we draw to leave room for an outline
     */
    private static final int INSET = 10;

    /**
     * horizontal space between pins
     */
    private static final int INTER_PIN_SPACING = 112;

    /**
     * thickness of line outlining pins
     */
    private static final int LINER = 10;

    /**
     * how long each pin is
     */
    private static final int PIN_LENGTH = 265;

    /**
     * how wide each pin is
     */
    private static final int PIN_WIDTH = 40;

    /**
     * how much shorter a short unused pin is
     */
    private static final int SHORTER = 30;

    /**
     * how wide the white stripes are
     */
    private static final int STRIPE_WIDTH = 15;

    /**
     * height of a jog in he connector outline
     */
    private static final int V_JOG = 200;

    /**
     * improved blue
     */
    private static final Color BLUE = new Color( 0x1e90ff );

    /**
     * improved green
     */
    private static final Color GREEN = new Color( 0x32cd32 );

    /**
     * improved orange
     */
    private static final Color ORANGE = new Color( 0xff8c00 );

    RJ45Female()
        {
        super( 1000, 910 );
        }

    /**
     * draw one pin
     *
     * @param g2d       graphics context
     * @param pinNumber pin number 1 to 8
     * @param color     What colour to draw the pin
     * @param isStriped true if pin should be striped
     * @param isUsed    true if pin is used.  false if not used (shown short)
     */
    private static void drawPin( Graphics2D g2d, int pinNumber, Color color, boolean isStriped, boolean isUsed )
        {
        g2d.setColor( color );
        int x = ( pinNumber - 1 ) * INTER_PIN_SPACING + FIRST_PIN;
        // draw unused pins short
        Rectangle pinOutline = new Rectangle( x - PIN_WIDTH / 2,
                isUsed ? INSET : INSET + SHORTER,
                PIN_WIDTH,
                PIN_LENGTH - ( isUsed ? INSET : INSET + SHORTER ) );
        g2d.fill( pinOutline );
        if ( isStriped )
            {
            Shape oldClip = g2d.getClip();
            g2d.clip( pinOutline );
            g2d.setColor( Color.WHITE );
            g2d.setStroke( new BasicStroke( STRIPE_WIDTH ) );
            for ( int i = 0; i < 3; i++ )
                {
                g2d.drawLine( x - PIN_WIDTH / 2,
                        PIN_LENGTH - i * 100,
                        x + PIN_WIDTH / 2, PIN_LENGTH - i * 100 - 100 );
                }
            g2d.setClip( oldClip );
            }
        g2d.setStroke( new BasicStroke( LINER ) );
        g2d.setColor( OUTLINE_COLOR );
        g2d.draw( pinOutline );
        }

    /**
     * draw a RJ11Female
     *
     * @param g where to paint
     */
    public void paintComponent( Graphics g )
        {
        super.paintComponent( g );
        final Graphics2D g2d = ( Graphics2D ) g;
        final int xCenter = 500;
        final int xRight = 1000;
        final int yCenter = 500;
        //  whole stepped connector
        g2d.setColor( HOLE_COLOR );
        Shape rj11Outline = new Polygon( new int[] {
                INSET,
                INSET,
                H_JOG,
                H_JOG,
                H_JOG * 2,
                H_JOG * 2,
                xRight - H_JOG * 2,
                xRight - H_JOG * 2,
                xRight - H_JOG,
                xRight - H_JOG,
                xRight - INSET,
                xRight - INSET },
                new int[] {
                        INSET,
                        yCenter,
                        yCenter,
                        yCenter + V_JOG,
                        yCenter + V_JOG,
                        yCenter + V_JOG * 2,
                        yCenter + V_JOG * 2,
                        yCenter + V_JOG,
                        yCenter + V_JOG,
                        yCenter,
                        yCenter,
                        INSET }
                , 12
        );
        g2d.fill( rj11Outline );
        g2d.setStroke( new BasicStroke( LINER ) );
        g2d.setColor( OUTLINE_COLOR );
        g2d.draw( rj11Outline );
        g2d.setFont( new Font( "Dialog", Font.PLAIN, 135 ) );
        g2d.setColor( OUTLINE_COLOR );
        drawCenteredString( g2d, "1 2 3 4 5 6 7 8", xCenter, 370 );
        g2d.setColor( OUTLINE_COLOR );
        drawPin( g2d, 1, ORANGE, false /* not striped */, true /* used */ );
        drawPin( g2d, 2, ORANGE, true /*  striped */, true /* used */ );
        drawPin( g2d, 3, GREEN, false /* not striped */, true /* used */ );
        drawPin( g2d, 4, BLUE, true /*  striped */, false /* not used */ );
        drawPin( g2d, 5, BLUE, false /* not striped */, false /* not used */ );
        drawPin( g2d, 6, GREEN, true /*  striped */, true /* used */ );
        drawPin( g2d, 7, Color.BLACK, false /* not striped */, false /* not used */ );
        drawPin( g2d, 8, Color.BLACK, true /*  striped */, false /* not used */ );
        }
    }
